##############################################################################
# This file evaluates the absorbed pathogen load for each pedestrian in each
# scenario. (choir rehearsal, i.e. miller-2020-life).
# Date: 2022-07-14
# Author: Simon Rahn
##############################################################################

import pandas as pd
import numpy as np
import math
import os

# insert at 1, 0 is the script path (or '' in REPL)
import sys
sys.path.insert(1, '..\\')
from plotSettingsPlosOne import *


if __name__ == '__main__':

    pathToOutputSubdir = "..\\..\\output\\choirRehearsal\\condM\\"

    # read data
    dumpedFileName = "pathogenLoadPerPedPerSimulation.pkl"
    dumpedFilePath = os.path.join(pathToOutputSubdir, dumpedFileName)

    if os.path.isfile(dumpedFilePath):
        pathogenLoadPerPedPerSimulation = pd.read_pickle(dumpedFilePath)
    else:
        dictionary = {}
        row = 0
        infectiousAgentId = 1

        for root, subdirs, _ in os.walk(pathToOutputSubdir):
            for subdir in subdirs:
                seed = subdir[-3:]
                file = "pathogenLoad.txt"
                pathToOutputFile = os.path.join(root, subdir, file)

                data = pd.read_csv(pathToOutputFile, sep=" ")
                dataWithoutInfectiousAgent = data[data['pedestrianId'] != infectiousAgentId]
                dictionary.update({seed: dataWithoutInfectiousAgent['pathogenLoad-PID2'].transpose().to_list()})

            # create dataframe that contains the absorbed pathogen load for each pedestrian (column) and simulation/seed (row)
            pathogenLoadPerPedPerSimulation = pd.DataFrame(dictionary)
            pathogenLoadPerPedPerSimulation = pathogenLoadPerPedPerSimulation.transpose()

            pd.to_pickle(pathogenLoadPerPedPerSimulation, dumpedFilePath)

    # additional data
    xMeasured = [32, 51]  # number of secondary cases [confirmed, probable] in hamner-life-2020

    # plot settings
    figsize = [figWidth/1.95, figHeight + 0.1]
    xLabel = "Number of virtual persons"
    yLabel = "Counts"
    adjustParam = {'left': 0.193, 'bottom': 0.25, 'right': 0.955, 'top': 0.957}

    # histogram settings
    binWidth = 1
    binsStart = 0.5
    binsEnd = 62.5
    nBins = math.floor((binsEnd - binsStart) / binWidth) + 1
    bins = np.linspace(start=binsStart, stop=binsEnd, num=nBins, endpoint=True)

    # common save options
    saveFig = True
    figDirPath = os.path.join(".")

    ##########################
    # histogram
    ##########################
    fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
    exposure3200 = pathogenLoadPerPedPerSimulation[pathogenLoadPerPedPerSimulation >= 3200].count(axis=1)
    exposure1000 = pathogenLoadPerPedPerSimulation[pathogenLoadPerPedPerSimulation >= 1000].count(axis=1)
    ax.hist(exposure3200, bins=bins, density=False, alpha=0.5, label='$D\geq3200$')
    ax.hist(exposure1000, bins=bins, density=False, alpha=0.5, label='$D\geq1000$')

    # plot line instead of bins
    # n,x,_ = plt.hist(maxCountsPerSimulation['E'], bins=bins, density=False)
    # binCenters = 0.5 * (x[1:] + x[:-1])
    # ax.plot(binCenters, n)

    # range of confirmed and probable cases
    yMin = 0
    yMax = 15
    ax.plot([xMeasured[0],xMeasured[0]], [0, yMax], color='k', ls='dotted')

    ax.set_xlabel(xLabel)
    ax.set_xlim([5, 40])

    ax.set_ylabel(yLabel)
    ax.set_ylim([yMin, yMax])
    ax.set_yticks(np.linspace(start=yMin, stop=yMax, num=4, endpoint=True))

    # ax.yaxis.set_major_locator(MaxNLocator(integer=True))

    # remove box around plot
    # ax.spines['top'].set_visible(False)
    # ax.spines['right'].set_visible(False)

    plt.subplots_adjust(left=adjustParam['left'], bottom=adjustParam['bottom'], right=adjustParam['right'], top=adjustParam['top'])

    plt.legend(framealpha=1)

    # save figure
    if (saveFig):
        figFileName = "histInfectionStatus"
        plt.savefig(os.path.join(figDirPath, figFileName + ".pdf"))
        plt.savefig(os.path.join(figDirPath, figFileName + ".eps"))
        plt.savefig(os.path.join(figDirPath, figFileName + ".tif"))
    else:
        plt.show()

    print('Finished.')
