##############################################################################
# This file evaluates the absorbed pathogen load for each pedestrian in each
# scenario (queue scenario).
# Date: 2022-07-14
# Author: Simon Rahn
##############################################################################
import pickle

import matplotlib as mpl
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import os


# insert at 1, 0 is the script path (or '' in REPL)
import sys
sys.path.insert(1, '..\\')
from plotSettingsPlosOne import *



if __name__ == '__main__':

    pathToOutputSubdir = "..\\..\\output\\queue\\parameter_variation\\"

    # read data
    dumpedFileName = "pathogenLoadPerPedPerSimulation.pkl"
    dumpedFilePath = os.path.join(pathToOutputSubdir, dumpedFileName)

    plotContactTimes = True
    dumpedFileName2 = 'contactTimesToInfectiousAgent.pkl'
    dumpedFilePath2 = os.path.join(pathToOutputSubdir, dumpedFileName2)
    if plotContactTimes & os.path.isfile(dumpedFilePath2):
        with open(dumpedFilePath2,'rb') as f:
            distanceToInfectiousAgentTimesSet = pickle.load(f)
            contactTimes = distanceToInfectiousAgentTimesSet[:,[0,1,2,3,5,6,7,8,9]] # exclude contact times between agent 5 and agent 5

    if os.path.isfile(dumpedFilePath):
        pathogenLoadPerPedPerSimulation = pd.read_pickle(dumpedFilePath)
    else:
        dictionary = {}
        row = 0
        infectiousAgentId = 5

        subdirs = os.listdir(pathToOutputSubdir)
        for subdir in subdirs:
            if os.path.isdir(os.path.join(pathToOutputSubdir, subdir)):
                parameterSet = subdir[-3:]
                subsubdirs = os.listdir(os.path.join(pathToOutputSubdir, subdir))
                file = "pathogenLoad.txt"
                pathToOutputFile = os.path.join(pathToOutputSubdir, subdir, subsubdirs[0], file)

                data = pd.read_csv(pathToOutputFile, sep=" ")
                dataWithoutInfectiousAgent = data[data['pedestrianId'] != infectiousAgentId]
                dictionary.update({parameterSet: dataWithoutInfectiousAgent['pathogenLoad-PID6'].transpose().to_list()})

        # create dataframe that contains the absorbed pathogen load for each pedestrian (column) and simulation/parameterSet (row)
        pathogenLoadPerPedPerSimulation = pd.DataFrame(dictionary)
        pathogenLoadPerPedPerSimulation = pathogenLoadPerPedPerSimulation.transpose()

        pd.to_pickle(pathogenLoadPerPedPerSimulation, dumpedFilePath)

    # convert dataframe to numpy array, store column/row info
    data = pathogenLoadPerPedPerSimulation.to_numpy()
    persSpaceWidth = np.array(pathogenLoadPerPedPerSimulation._get_axis(0).to_list()).astype(float)
    # persSpaceWidthLabels = ['$P=0.5$','$P=1.0$','$P=1.5$','$P=2.0$','$P=2.5$']
    persSpaceWidthLabels = ['A','B','C','D','E'] # alternative labels
    pedIds = np.array(pathogenLoadPerPedPerSimulation._get_axis(1).to_list()).astype(int)
    pedIdLabes = np.array(['1','2','3','4','6','7','8','9','10'])


    # common save options
    saveFig = True
    figDirPath = os.path.join(".")

    # common plt options
    barGroupWidth = 0.75
    adjustParam = {'left': 0.152, 'bottom': 0.225, 'right': 0.979, 'top': 0.88}
    ylim = [0, 5000]
    yticks = np.linspace(ylim[0],ylim[1], 6)


    ##########################
    # bar plot (ped id vs. degree of exposure)
    ##########################
    figsize = [figWidth/1.95, figHeight + 0.1]
    xLabel = "Position of the virtual person"
    yLabel = "Degree of exposure"
    nBars = len(persSpaceWidth)
    barWidth = barGroupWidth/nBars

    xticks = pedIds
    dx = 1 # distance between values of xticks

    color = plt.cm.Blues_r(np.linspace(0.2,0.8,nBars))
    mpl.rcParams['axes.prop_cycle'] = mpl.cycler(color=color)

    fig, ax = plt.subplots(figsize=figsize, dpi=dpi)

    tempPos = []
    # for i in range(nBars-1,-1,-1):
    for i in range(0, nBars):
        ax.bar(xticks+(i-(nBars-1)/2)*barWidth*dx, data[i, :], width=barWidth*dx, label=persSpaceWidthLabels[i], color=color[i])
        if (plotContactTimes):
            for agentPosition in xticks:
                contactTime = contactTimes[i, agentPosition] / 60
                if contactTime >= 10:
                    tempPos.append([agentPosition+(i-(nBars-1)/2)*barWidth*dx, data[i, agentPosition]])
                    # ax.text(agentPosition+(i-(nBars-1)/2)*barWidth*dx-barWidth, data[i, agentPosition] + 10, "$t_c\geq10\,$min", rotation=45, ha="left", va="bottom", fontsize=4)

    if len(tempPos)>0:
        textPos = [4.25, 4400]
        # ax.text(textPos[0], textPos[1], "contact time $\geq10\,$min", ha="center", va="bottom")
        ax.text(8.65, 4900, "contact time $\geq10\,$min", ha="right", va="top")
        for i in range(0, len(tempPos)):
            # ax.arrow(textPos[0], textPos[1], tempPos[i][0]-textPos[0], tempPos[i][1]-textPos[1], length_includes_head=True, width=0.0001)
            ax.annotate("", xy=tempPos[i], xytext=textPos, arrowprops=dict(arrowstyle='->', shrinkA=0, shrinkB=1, lw=1))


    ax.set_xlabel(xLabel)
    ax.set_xticks(xticks)
    ax.set_ylim([-1,9])
    ax.set_xticklabels(pedIdLabes)
    ax.set_ylabel(yLabel)
    ax.set_ylim(ylim)
    ax.set_yticks(yticks)
    ax.ticklabel_format(axis="y", style="sci", scilimits=(0,0))
    ax.ticklabel_format(axis="y", useMathText=True)

    plt.subplots_adjust(left=adjustParam['left'], bottom=adjustParam['bottom'], right=adjustParam['right'], top=adjustParam['top'])

    plt.legend(framealpha=1, ncol=1, columnspacing=0.75, labelspacing=0.2,handlelength=0.5,handletextpad=0.2)

    # save figure
    if (saveFig):
        figFileName = "barPlot_pedId-degreeOfExposure"
        plt.savefig(os.path.join(figDirPath, figFileName + ".pdf"))
        plt.savefig(os.path.join(figDirPath, figFileName + ".eps"))
        plt.savefig(os.path.join(figDirPath, figFileName + ".tif"))
    else:
        plt.show()

    plt.close(fig)


    ##########################
    # bar plot (paramset vs. degree of exposure)
    ##########################
    # figsize = [figWidth/1.95, figHeight + 0.1]
    # xLabel = "pedPotentialPersonalSpaceWidth"
    # yLabel = "Degree of exposure"
    # nBars = len(pedIds)
    # barWidth = barGroupWidth/nBars
    #
    # xticks = persSpaceWidth
    # dx = 0.5 # distance between xticks
    #
    # color = plt.cm.Blues_r(np.linspace(0.2,0.8,nBars))
    # mpl.rcParams['axes.prop_cycle'] = mpl.cycler(color=color)
    #
    # fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
    #
    # for i in range(0,nBars):
    #     ax.bar(xticks+(i-(nBars-1)/2)*barWidth*dx, data[:, i], width=barWidth*dx, label=pedIdLabes[i])
    #
    # ax.set_xlabel(xLabel)
    # ax.set_xticks(xticks)
    # ax.set_xticklabels(xticks.astype(str))
    # ax.set_ylabel(yLabel)
    # ax.set_ylim(ylim)
    # ax.set_yticks(yticks)
    # ax.ticklabel_format(axis="y", style="sci", scilimits=(0,0))
    # ax.ticklabel_format(axis="y", useMathText=True)
    #
    # plt.subplots_adjust(left=adjustParam['left'], bottom=adjustParam['bottom'], right=adjustParam['right'], top=adjustParam['top'])
    #
    # plt.legend(framealpha=1, ncol=3, columnspacing=0.75, labelspacing=0.2,handlelength=0.5,handletextpad=0.2)
    #
    # # save figure
    # if (saveFig):
    #     figFileName = "barPlot_persSpaceWidth-degreeOfExposure"
    #     plt.savefig(os.path.join(figDirPath, figFileName + ".pdf"))
    #     plt.savefig(os.path.join(figDirPath, figFileName + ".eps"))
    #     plt.savefig(os.path.join(figDirPath, figFileName + ".tif"))
    # else:
    #     plt.show()
    #
    # plt.close(fig)

    print('Finished.')
